﻿using System;
using System.Linq;
using System.Threading.Tasks;
using Bots.Grind;
using Buddy.Coroutines;
using CommonBehaviors.Actions;
using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.CommonBot.POI;
using Styx.Pathing;
using Styx.TreeSharp;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace PocketFelSpreader
{
    public class Core : BotBase
    {
        public Composite _root;
        public override string Name => "Pocket Fel Spreader";

        public override PulseFlags PulseFlags => PulseFlags.All;


        public override Composite Root
        {
            get { return _root ?? (_root = new ActionRunCoroutine(ctx => MainTask())); }
        }

        //<WoWUnit Name = "Dread Infiltrator" Entry="112315" X="-8828.072" Y="627.9253" Z="94.50502" />
        public static WoWUnit DreadInfiltrator => ObjectManager.GetObjectsOfType<WoWUnit>().
            FirstOrDefault(
                u =>
                    u.IsValid && u.IsAlive && (u.Entry == 112315 || u.Entry == 112198 || u.Entry == 112527) &&
                    u.FactionId == 2780);

        public static WoWUnit Doomsayer => ObjectManager.GetObjectsOfType<WoWUnit>().OrderBy(s => s.Distance).
            FirstOrDefault(
                u =>
                    u.IsValid && !Blacklist.Contains(u, BlacklistFlags.Interact) &&
                    (u.Entry == 112198 || u.Entry == 112527));

        //<Aura Name="Pamphlet" SpellId="223505" />
        public static bool Pamphlet => StyxWoW.Me.HasAura(223505);

        public static async Task<bool> MainTask()
        {
            var dread = DreadInfiltrator;
            var doom = Doomsayer;

            if ((StyxWoW.Me.IsDead || StyxWoW.Me.IsGhost) && await
                LevelBot.CreateDeathBehavior().ExecuteCoroutine())
                return true;

            if (StyxWoW.Me.Combat && await
                LevelBot.CreateCombatBehavior().ExecuteCoroutine())
                return true;

            if (await LootTask())
                return true;

            if (dread != null)
            {
                BotPoi.Current.Guid = dread.Guid;
                BotPoi.Current.Type = PoiType.Kill;
                BotPoi.Current.Location = dread.Location;
                BotPoi.Current.Entry = dread.Entry;
                BotPoi.Current.Name = dread.Name;

                return await DoGossip(dread) && await LevelBot.CreateCombatBehavior().ExecuteCoroutine();
            }

            if (Pamphlet)
            {
                if (StyxWoW.Me.IsAFKFlagged)
                {
                    StyxWoW.ResetAfk();
                    Log("Reset Afk");
                }
                var duration = GetAuraDuration(223505);
                Status($"Waiting for Pamphlet debuff to expire in {duration.Minutes}min {duration.Seconds}sec");
                await Coroutine.Sleep(5000);
                BotPoi.Clear();
                return true;
            }

            if (doom != null)
                await DoGossip(doom);


            return true;
        }

        public static async Task<bool> TrackDemons()
        {
            var me = StyxWoW.Me;
            var trackDemons = 19878;
            var spectralSight = 188501;

            if (me.Class == WoWClass.Hunter && !me.HasAura(trackDemons))
                return await CastSpell(trackDemons);

            if (me.Class == WoWClass.DemonHunter && !me.HasAura(spectralSight))
                return await CastSpell(spectralSight);

            //<WoWItem Name="Elixir of Detect Demon" Entry="9233" />
            var elixer = StyxWoW.Me.BagItems.FirstOrDefault(b => b.Entry == 9233);
            //<Aura Name="Detect Demon" SpellId="11407" />
            if (!StyxWoW.Me.HasAura(11407) && elixer != null)
            {
                Log($"Using {elixer}");              
                return elixer.Use(); ;
            }

            return true;
        }

        public static async Task<bool> LootTask()
        {
            if (LootFrame.Instance.IsVisible)
            {
                LootFrame.Instance.LootAll();
                Log($"Looting {LootFrame.Instance.FrameName}");
                await CommonCoroutines.SleepForRandomUiInteractionTime();
                return true;
            }

            var loot =
                ObjectManager.GetObjectsOfType<WoWUnit>().FirstOrDefault(u => u.IsValid && u.Lootable && u.CanLoot);

            if (loot == null || !loot.IsValid) return false;

            if (!loot.WithinInteractRange)
            {
                var move = await CommonCoroutines.MoveTo(loot.Location, loot.Name);

                Status($"Moving to loot {loot}");
                if (move == MoveResult.PathGenerationFailed)
                    Blacklist.Add(loot, BlacklistFlags.Loot, TimeSpan.FromSeconds(30), "PathGenerationFailed");

                return true;
            }

            loot.Interact();
            await CommonCoroutines.SleepForRandomUiInteractionTime();
            return true;
        }

        public static async Task<bool> DoGossip(WoWUnit unit)
        {
            if (unit == null || !unit.IsValid) return false;

            if (!unit.WithinInteractRange && Flightor.CanFly)
            {
                if (!Flightor.MountHelper.Mounted)
                {
                    Flightor.MountHelper.MountUp();
                    await Coroutine.Sleep(5000);
                }
                Flightor.MoveTo(unit.Location);
                Status($"Flying to {unit}");
                return true;
            }

            if (!unit.WithinInteractRange)
            {
                var move = await CommonCoroutines.MoveTo(unit.Location, unit.Name);

                Status($"Moving to {unit}");
                if (move == MoveResult.PathGenerationFailed)
                    Blacklist.Add(unit, BlacklistFlags.Interact, TimeSpan.FromSeconds(30), "PathGenerationFailed");

                return true;
            }

            if (Flightor.MountHelper.Mounted)
            {
                Flightor.MountHelper.Dismount();
                await Coroutine.Sleep(5000);
            }

            if (!await TrackDemons())
            {
                Log("Waiting to track Demons");
                await Coroutine.Sleep(5000);
                return true;
            }


            var gossip = GossipFrame.Instance;

            if (!gossip.IsVisible)
            {
                unit.Interact();
                await CommonCoroutines.SleepForRandomUiInteractionTime();
                return true;
            }

            foreach (var g in gossip.GossipOptionEntries)
            {
                Log($"Selecting {g.Text}");
                gossip.SelectGossipOption(g.Index);
                await CommonCoroutines.SleepForRandomUiInteractionTime();
                return true;
            }


            gossip.Close();
            Blacklist.Add(unit.Guid, BlacklistFlags.Interact, TimeSpan.FromMinutes(10), "No Gossip Options");
            await Coroutine.Sleep(5000);
            return true;
        }


        public static async Task<bool> CastSpell(int spellId)
        {
            if (!SpellManager.HasSpell(spellId)) return false;

            if (!SpellManager.CanCast(spellId)) return false;

            if (!SpellManager.CastSpellById(spellId)) return false;

            Log($"Casting {spellId}");
            await CommonCoroutines.SleepForLagDuration();
            return true;
        }

        public static TimeSpan GetAuraDuration(int auraId)
        {
            if (!StyxWoW.Me.HasAura(auraId)) return TimeSpan.Zero;

            var aura = StyxWoW.Me.Auras.FirstOrDefault(a => a.Value.SpellId == auraId);

            return aura.Value?.TimeLeft ?? TimeSpan.Zero;
        }

        public static void Log(string text)
        {
            Logging.Write(LogLevel.Normal, text);
        }

        public static void Status(string text)
        {
            TreeRoot.StatusText = text;
        }
    }
}